# /packages/intranet-reporting-finance/www/finance-cash-flow.tcl
#
# Copyright (C) 2003 - 2009 ]project-open[
#
# All rights reserved. Please check
# https://www.project-open.com/ for licensing details.


ad_page_contract {
    Cash flow report - the list of financial objects 
    order by date, and weighted +/- depending on 
    incoming vs. outgoing money.
} {
    { report_start_date "" }
    { report_end_date "" }
    { report_start_amount "" }
    { report_burn_rate "" }
    { report_burn_rate_day_of_month "" }
    { report_cost_center_id "" }
    { level_of_detail 3 }
    { output_format "html" }
    { number_locale "" }
}

# ------------------------------------------------------------
# Security

# Label: Provides the security context for this report
# because it identifies unquely the report's Menu and
# its permissions.
set menu_label "reporting-finance-cash-flow"
set current_user_id [ad_maybe_redirect_for_registration]
set read_p [db_string report_perms "
	select	im_object_permission_p(m.menu_id, :current_user_id, 'read')
	from	im_menus m
	where	m.label = :menu_label
" -default 'f']

if {![string equal "t" $read_p]} {
    ad_return_complaint 1 "<li>
[lang::message::lookup "" intranet-reporting.You_dont_have_permissions "You don't have the necessary permissions to view this page"]"
    return
}

set page_title "Cash Flow"
set context_bar [im_context_bar $page_title]
set context ""


# Check that Start & End-Date have correct format
if {"" != $report_start_date && ![regexp {^[0-9][0-9][0-9][0-9]\-[0-9][0-9]\-[0-9][0-9]$} $report_start_date]} {
    ad_return_complaint 1 "Start Date doesn't have the right format.<br>
    Current value: '$report_start_date'<br>
    Expected format: 'YYYY-MM-DD'"
}

if {"" != $report_end_date && ![regexp {^[0-9][0-9][0-9][0-9]\-[0-9][0-9]\-[0-9][0-9]$} $report_end_date]} {
    ad_return_complaint 1 "End Date doesn't have the right format.<br>
    Current value: '$report_end_date'<br>
    Expected format: 'YYYY-MM-DD'"
}


# ------------------------------------------------------------
# Defaults

set rowclass(0) "roweven"
set rowclass(1) "rowodd"

set days_in_past 7

set default_currency [im_parameter -package_id [im_package_cost_id] "DefaultCurrency" "" "EUR"]
set cur_format [im_l10n_sql_currency_format]
set date_format [im_l10n_sql_date_format]
set locale [lang::user::locale]
if {"" == $number_locale} { set number_locale $locale  }

# Maxlevel is 4. Normalize in order to show the right drop-down element
if {$level_of_detail > 3} { set level_of_detail 3 }

set max_project_name_len 20


set company_url "/intranet/companies/view?company_id="
set project_url "/intranet/projects/view?project_id="
set user_url "/intranet/users/view?user_id="
set invoice_url "/intranet-invoices/view?invoice_id="
set this_url [export_vars -base "/intranet-cust-cosine/reports/cash-flow" {report_start_amount report_burn_rate report_burn_rate_day_of_month} ]

set burn_rate_l10n "[lang::message::lookup "" intranet-reporting.Burn_Rate "Burn Rate"] "


db_1row todays_date "
select
	to_char(sysdate::date - :days_in_past::integer, 'YYYY') as todays_year,
	to_char(sysdate::date - :days_in_past::integer, 'MM') as todays_month,
	to_char(sysdate::date - :days_in_past::integer, 'DD') as todays_day
from dual
"

if {"" == $report_start_date} { set report_start_date "$todays_year-01-01" }

db_1row report_end_date "
select
	to_char(to_date(:report_start_date, 'YYYY-MM-DD') + 365::integer, 'YYYY') as end_year,
	to_char(to_date(:report_start_date, 'YYYY-MM-DD') + 365::integer, 'MM') as end_month,
	to_char(to_date(:report_start_date, 'YYYY-MM-DD') + 365::integer, 'DD') as end_day
from dual
"

if {"" eq $report_end_date} { set report_end_date "$end_year-$end_month-01" }


# Estimate the burn rate based on the number of employees and their hourly_cost
if {"" eq $report_burn_rate} { 
    set default_hourly_cost [parameter::get_from_package_key -package_key intranet-costs -parameter DefaultTimesheetHourlyCost -default 50]
    set hours_per_day [parameter::get_from_package_key -package_key intranet-timesheet2 -parameter TimesheetHoursPerDay -default 8]
    set average_working_days_per_month [parameter::get_from_package_key -package_key intranet-timesheet2 -parameter AverageWorkingDaysPerMonth -default 20]
    set report_burn_rate_sql "
 	select	round(sum(
			coalesce(e.hourly_cost, :default_hourly_cost)
			* :hours_per_day 
			* :average_working_days_per_month
		))
	from	im_employees e
	where
		e.employee_id in (
			select	member_id
			from	group_distinct_member_map
			where	group_id in (select group_id from groups where group_name = 'Employees')       
		) and
		'approved' = (
			select min(mr.member_state)
			from acs_rels r, membership_rels mr 
			where r.object_id_two = e.employee_id and r.object_id_one = -2 and r.rel_id = mr.rel_id
		)
    "
    # ad_return_complaint 1 [im_ad_hoc_query -format html $report_burn_rate_sql]
    set report_burn_rate [db_string report_burn_rate $report_burn_rate_sql]
}

if {"" eq $report_burn_rate_day_of_month} {
   set report_burn_rate_day_of_month [parameter::get_from_package_key -package_key intranet-cost -parameter DefaultBurnRateDayOfMonth -default 15]
}
set burn_dom [expr $report_burn_rate_day_of_month - 1]


if {"" eq $report_start_amount} {
   set report_start_amount [parameter::get_from_package_key -package_key intranet-cost -parameter DefaultBurnRateStartAmount -default 1000000]
}

# ------------------------------------------------------------
# Conditional SQL Where-Clause
#

set criteria [list]

if {"" ne $report_cost_center_id && 0 ne $report_cost_center_id} {
    lappend criteria "c.cost_center_id in ([join [im_sub_cost_center_ids $report_cost_center_id] ", "])"
}

set where_clause [join $criteria " and\n            "]
if {![empty_string_p $where_clause]} {
    set where_clause " and $where_clause"
}


# ------------------------------------------------------------
# Define the report - SQL, counters, headers and footers 
#

set inner_sql "
-- Monthly burn rate with extra payment in May
select
	null as cost_id,
	3714 as cost_type_id,
	3802 as cost_status_id,
	null as cost_center_id,
	0 as material_id,
	:burn_rate_l10n||(im_month_enumerator + :burn_dom::integer) as cost_nr,
	:burn_rate_l10n||(im_month_enumerator + :burn_dom::integer) as cost_name,
	(im_month_enumerator + :burn_dom::integer) as effective_date,
	null as customer_id,
	null as provider_id,
	null as root_sortkey,
	0 as item_id,
	0 as item_sort_order,
	:burn_rate_l10n||(im_month_enumerator + :burn_dom::integer) as item_name,
	:report_burn_rate as amount,
	:report_burn_rate as amount_converted
from
	im_month_enumerator(:report_start_date, :report_end_date)
where
	(im_month_enumerator + :burn_dom::integer) >= to_date(:report_start_date, 'YYYY-MM-DD') and
	(im_month_enumerator + :burn_dom::integer) < to_date(:report_end_date, 'YYYY-MM-DD')

UNION

-- Financial Documents with im_invoice_items
select
	c.cost_id,
	c.cost_type_id,
	c.cost_status_id,
        c.cost_center_id,                                                                                                                       
        ii.item_material_id as material_id,                                                                                                     
	c.cost_nr,
	c.cost_name,
	c.effective_date,
	c.customer_id,
	c.provider_id,
	tree_root_key(p.tree_sortkey) as root_sortkey,
        ii.item_id,                                                                                                                             
        ii.sort_order as item_sort_order,                                                                                                       
        substring(ii.item_name for 60) as item_name,                                                                                            
        ii.item_units * ii.price_per_unit as amount,                                                                                            
        round((                                                                                                                                 
                ii.item_units * ii.price_per_unit *
                im_exchange_rate(c.effective_date::date, c.currency, :default_currency)) :: numeric
        , 2) as amount_converted
from
	im_invoices i,
	im_invoice_items ii,
	im_costs c
	LEFT OUTER JOIN im_projects p ON (c.project_id = p.project_id)
where	c.cost_id = i.invoice_id and
	c.cost_id = ii.invoice_id and
	(	c.cost_type_id in ([join [im_sub_categories [im_cost_type_invoice]] ","])
	OR	c.cost_type_id in ([join [im_sub_categories [im_cost_type_bill]] ","])
        OR      c.cost_type_id in ([join [im_sub_categories [im_cost_type_quote]] ","])
        OR      c.cost_type_id in ([join [im_sub_categories [im_cost_type_po]] ","])
	) and
	c.cost_status_id not in ([join [im_sub_categories [im_cost_status_deleted]] ","]) and
	c.effective_date >= to_date(:report_start_date, 'YYYY-MM-DD') and
	c.effective_date < to_date(:report_end_date, 'YYYY-MM-DD')
	$where_clause

UNION

-- Expense Bundles are different - no im_invoice_items
select
	c.cost_id,
	c.cost_type_id,
	c.cost_status_id,
	c.cost_center_id,
	0 as material_id,
	c.cost_nr,
	c.cost_name,
	c.effective_date,
	c.customer_id,
	c.provider_id,
	tree_root_key(p.tree_sortkey) as root_sortkey,
	0 as item_id,
	0 as item_sort_order,
	c.cost_name as item_name,
	c.amount as amount,
	round((
	c.amount  * 
		im_exchange_rate(c.effective_date::date, c.currency, :default_currency)) :: numeric
	,2) as amount_converted
from
	im_costs c
	LEFT OUTER JOIN im_projects p ON (c.project_id = p.project_id)
where	c.cost_type_id in ([join [im_sub_categories [im_cost_type_expense_bundle]] ","]) and
	c.cost_status_id not in ([join [im_sub_categories [im_cost_status_deleted]] ","]) and
	c.effective_date >= to_date(:report_start_date, 'YYYY-MM-DD') and
	c.effective_date < to_date(:report_end_date, 'YYYY-MM-DD')
	$where_clause

"

set sql "
select
	c.*,
	im_cost_center_code_from_id(c.cost_center_id) as cost_center_code,
	to_char(c.effective_date, :date_format) as effective_date_formatted,
	to_char(c.effective_date, 'YYYY-MM') as effective_month,
	im_category_from_id(c.cost_type_id) as cost_type,
	im_category_from_id(c.cost_status_id) as cost_status,
	(select material_name from im_materials where material_id = c.material_id) as material_name,
	cust.company_id as customer_id,
	cust.company_path as customer_nr,
	cust.company_name as customer_name,
	prov.company_id as provider_id,
	prov.company_path as provider_nr,
	prov.company_name as provider_name,
	main_p.project_id as project_id,
	main_p.project_nr as project_nr,
	substring(main_p.project_name for :max_project_name_len) as project_name,
	main_p.project_status_id,
	coalesce(CASE WHEN c.cost_type_id in ([join [im_sub_categories 3700] ","]) THEN c.amount_converted END, 0) as invoice_amount,
	coalesce(CASE WHEN c.cost_type_id in ([join [im_sub_categories 3702] ","]) THEN c.amount_converted END, 0) as quote_amount,
	coalesce(CASE WHEN c.cost_type_id in ([join [im_sub_categories 3704] ","]) THEN -c.amount_converted END, 0) as bill_amount,
	coalesce(CASE WHEN c.cost_type_id in ([join [im_sub_categories 3706] ","]) THEN -c.amount_converted END, 0) as po_amount,
	coalesce(CASE WHEN c.cost_type_id in ([join [im_sub_categories 3714] ","]) THEN -c.amount_converted END, 0) as burn_amount,
	coalesce(CASE WHEN c.cost_type_id in ([join [im_sub_categories 3722] ","]) THEN -c.amount_converted END, 0) as expense_amount
from
	($inner_sql) c
	left outer join im_companies cust on (c.customer_id = cust.company_id)
	left outer join im_companies prov on (c.provider_id = prov.company_id)
	left outer join im_projects main_p on (c.root_sortkey = main_p.tree_sortkey)
order by
	c.effective_date
"


# Report Body
set report_def [list \
    group_by cost_id \
    header {
			"<nobr>$effective_date_formatted</nobr>"
			"<a href=$company_url$customer_id target=_>$customer_name</a>"
			"<a href=$company_url$provider_id target=_>$provider_name</a>"
			"$cost_center_code"
			"<nobr><a href=$project_url$project_id target=_>$project_name</a></nobr>"
			"<nobr><a href=$invoice_url$cost_id target=_>$cost_name</a></nobr>"
			"<nobr>$material_name</nobr>"
			"<nobr>$item_name</nobr>"
			"<nobr>$cost_type</nobr>"
			"<nobr>$cost_status</nobr>"
			"<div align=right><nobr>$invoice_amount_pretty</nobr></div>"
			"<div align=right><nobr>$quote_amount_pretty</nobr></div>"
			"<div align=right><nobr>$bill_amount_pretty</nobr></div>"
			"<div align=right><nobr>$po_amount_pretty</nobr></div>"
			"<div align=right><nobr>$expense_amount_pretty</nobr></div>"
			"<div align=right><nobr>$burn_amount_pretty</nobr></div>"
			"<div align=right><nobr>$amount_sum_pretty</nobr></div>"
			"<div align=right><nobr>$amount_aggregated_pretty</nobr></div>"
    } \
    content {} \
    footer {} \
]

set invoice_total 0
set quote_total 0
set bill_total 0
set po_total 0
set amount_aggregated 0


# Global header/footer
set header0 {"Date" "Customer" "Provider" "CC" "Project" "Cost Name" "Material" "Line Name" "Type" "Status" "Invoice" "Quote" "Bill" "PO" "Travel Expenses" "Burn Rate" "Amount" "Aggregated"}
set footer0 {}

set amount_counter [list \
        pretty_name "Amount Aggregated" \
        var amount_aggregated \
        reset 0 \
        expr "\$amount_sum+0" \
]

set counters [list \
        $amount_counter \
]


# ------------------------------------------------------------
# Constants
#

set levels {1 "Customer Only" 2 "Customer+Date" 3 "All Details"} 

# ------------------------------------------------------------
# Start formatting the page header
#

# Write out HTTP header, considering CSV/MS-Excel formatting
im_report_write_http_headers -output_format $output_format -report_name "finance-cash-flow"

# Add the HTML select box to the head of the page
switch $output_format {
    html {
	ns_write "
	[im_header]
	[im_navbar reporting]
	<form>
               [export_vars -form {customer_id}]
		<table border=0 cellspacing=1 cellpadding=1>
<!--
		<tr>
		  <td class=form-label>Level of Details</td>
		  <td class=form-widget>
		    [im_select -translate_p 0 level_of_detail $levels $level_of_detail]
		  </td>
		</tr>
-->
		<tr>
		  <td class=form-label>Start Amount ($default_currency)</td>
		  <td class=form-widget>
		    <input type=textfield name=report_start_amount value=$report_start_amount>
		  </td>
		</tr>
		<tr>
		  <td class=form-label>Monthly Burn Rate ($default_currency)</td>
		  <td class=form-widget>
		    <input type=textfield name=report_burn_rate value=$report_burn_rate>
		  </td>
		</tr>
		<tr>
		  <td class=form-label>Start Date</td>
		  <td class=form-widget>
		    <input type=textfield name=report_start_date value=$report_start_date>
		  </td>
		</tr>
		<tr>
		  <td class=form-label>End Date</td>
		  <td class=form-widget>
		    <input type=textfield name=report_end_date value=$report_end_date>
		  </td>
		</tr>
                <tr>
                  <td class=form-label>Format</td>
                  <td class=form-widget>
                    [im_report_output_format_select output_format "" $output_format]
                  </td>
                </tr>
                <tr>
                  <td class=form-label><nobr>Number Format</nobr></td>
                  <td class=form-widget>
                    [im_report_number_locale_select number_locale $number_locale]
                  </td>
                </tr>
		<tr>
		  <td class=form-label></td>
		  <td class=form-widget><input type=submit value=Submit></td>
		</tr>
		</table>
	</form>
	<table border=0 cellspacing=2 cellpadding=2>\n"
    }
}

im_report_render_row \
    -output_format $output_format \
    -row $header0 \
    -row_class "rowtitle" \
    -cell_class "rowtitle"


set footer_array_list [list]
set last_value_list [list]
set class "rowodd"
db_foreach cash_flow_query $sql {

	# Ignore $quote_amount and $po_amount
	set amount_sum [expr $invoice_amount + $bill_amount + $expense_amount + $burn_amount]

	set invoice_amount_pretty [im_report_format_number $invoice_amount $output_format $number_locale]
	set quote_amount_pretty [im_report_format_number $quote_amount $output_format $number_locale]
	set bill_amount_pretty [im_report_format_number $bill_amount $output_format $number_locale]
	set po_amount_pretty [im_report_format_number $po_amount $output_format $number_locale]
	set expense_amount_pretty [im_report_format_number $expense_amount $output_format $number_locale]
	set burn_amount_pretty [im_report_format_number $burn_amount $output_format $number_locale]


	if {[string length $project_name] == $max_project_name_len} { set project_name "$project_name..." }

	im_report_display_footer \
	    -output_format $output_format \
	    -group_def $report_def \
	    -footer_array_list $footer_array_list \
	    -last_value_array_list $last_value_list \
	    -level_of_detail $level_of_detail \
	    -row_class $class \
	    -cell_class $class
	
	im_report_update_counters -counters $counters

	set amount_sum_pretty [im_report_format_number $amount_sum $output_format $number_locale]
	set amount_aggregated_pretty [im_report_format_number [expr round(100.0 * ($report_start_amount + $amount_aggregated)) / 100.0] $output_format $number_locale]

	
	set last_value_list [im_report_render_header \
	    -output_format $output_format \
	    -group_def $report_def \
	    -last_value_array_list $last_value_list \
	    -level_of_detail $level_of_detail \
	    -row_class $class \
	    -cell_class $class
        ]

        set footer_array_list [im_report_render_footer \
	    -output_format $output_format \
	    -group_def $report_def \
	    -last_value_array_list $last_value_list \
	    -level_of_detail $level_of_detail \
	    -row_class $class \
	    -cell_class $class
        ]
}

im_report_display_footer \
    -output_format $output_format \
    -group_def $report_def \
    -footer_array_list $footer_array_list \
    -last_value_array_list $last_value_list \
    -level_of_detail $level_of_detail \
    -display_all_footers_p 1 \
    -row_class $class \
    -cell_class $class

im_report_render_row \
    -output_format $output_format \
    -row $footer0 \
    -row_class $class \
    -cell_class $class \
    -upvar_level 1


switch $output_format {
    html { ns_write "</table>\n[im_footer]\n" }
}


